using System;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Web;
using System.Data;
using System.Web.Caching;

public class DatabaseImageHandler : IHttpHandler {
    private const string CACHE_NAME = "DBIMAGE";

    #region IHttpHandler Members
    public bool IsReusable {
        get { return true; }
    }

    public void ProcessRequest(HttpContext context) {
        if(!context.User.Identity.IsAuthenticated) {
            //Forbidden
            context.Response.StatusCode = 403;
            context.Response.End();
            return;
        }

        string strId = context.Request.QueryString["id"];
        ImageCacheEntry cacheEntry = context.Cache[CACHE_NAME + strId] as ImageCacheEntry;
        if(cacheEntry == null) {
            int id;
            if(!int.TryParse(strId, out id)) {
                context.Response.StatusCode = 500;
                context.Response.End();
                return;
            }

            cacheEntry = new ImageCacheEntry();
            using(SqlConnection conn = new SqlConnection("Server=.; Database=AdventureWorks; Integrated Security=true")) {
                SqlCommand cmd = new SqlCommand("SELECT ThumbNailPhoto, ThumbnailPhotoFileName, ModifiedDate FROM Production.ProductPhoto WHERE ProductPhotoID=@photoId", conn);
                cmd.Parameters.Add(new SqlParameter("photoId", id));
                conn.Open();
                SqlDataReader reader = cmd.ExecuteReader();
                if(reader.Read()) {
                    cacheEntry.Filename = (string)reader["ThumbnailPhotoFileName"];
                    // We're only handling gifs right now
                    Debug.Assert(Path.GetExtension(cacheEntry.Filename) == ".gif");
                    cacheEntry.ImageData = (byte[])reader["ThumbNailPhoto"];
                    cacheEntry.LastModified = (DateTime)reader["ModifiedDate"];
                }
                reader.Close();
                conn.Close();
            }
            if(cacheEntry == null) {
                context.Response.StatusCode = 404;
                context.Response.End();
                return;
            }
            context.Cache.Insert(CACHE_NAME + id, cacheEntry, null, DateTime.Now.AddMinutes(30), Cache.NoSlidingExpiration);
        }

        // Check if we need to send the content
        string modifiedSinceHeader = context.Request.Headers["If-Modified-Since"];
        if(!string.IsNullOrEmpty(modifiedSinceHeader)) {
            DateTime ifModifiedSince;
            if(DateTime.TryParse(modifiedSinceHeader, out ifModifiedSince)) {
                TimeSpan timeDiff = ifModifiedSince - cacheEntry.LastModified;
                if(timeDiff.TotalSeconds < 1 && timeDiff.TotalSeconds > -1) {
                    context.Response.StatusCode = 304;
                    context.Response.End();
                    return;
                }
            }
        }

        // Set your caching
        context.Response.Cache.SetCacheability(HttpCacheability.Public);
        context.Response.Cache.SetExpires(DateTime.Now.AddDays(1));
        context.Response.Cache.SetLastModified(cacheEntry.LastModified);
        context.Response.ContentType = "image/gif";
        BinaryWriter writer = new BinaryWriter(context.Response.OutputStream);
        try {
            writer.Write(cacheEntry.ImageData);
        } finally {
            writer.Close();
        }
    }
    #endregion
}
