using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Web;

public class TextDrawingHandler : IHttpHandler {
    #region IHttpHandler Members
    public bool IsReusable {
        get { return true; }
    }

    public void ProcessRequest(HttpContext context) {
        // Set your caching
        context.Response.Cache.SetCacheability(HttpCacheability.Public);
        context.Response.Cache.SetExpires(DateTime.Now.AddSeconds(30));

        if(!context.User.Identity.IsAuthenticated) {
            //Forbidden
            context.Response.StatusCode = 403;
            context.Response.End();
            return;
        }

        // Substring to remove the leading "." in the extension
        string ext = Path.GetExtension(context.Request.Path).ToLowerInvariant().Substring(1);
        if(Array.Exists(new string[] { "jpg", "jpeg", "gif" }, delegate(string comparison) {
                                                                                             return comparison == ext;
                                                                                         })) {
            context.Response.ContentType = "image/" + ext;
        } else {
            context.Response.StatusCode = 500;
            context.Response.End();
            return;
        }

        string text = Path.GetFileNameWithoutExtension(context.Request.Path);
        Font verdanaBold = new Font("Verdana", 36, FontStyle.Bold);
        SizeF imageSize;
        using(Bitmap dummy = new Bitmap(1, 1))
        using(Graphics g = Graphics.FromImage(dummy)) {
            imageSize = g.MeasureString(text, verdanaBold);
        }
        int width = (int)Math.Ceiling(imageSize.Width);
        int height = (int)Math.Ceiling(imageSize.Height);
        using(Bitmap img = new Bitmap(width, height))
        using(Graphics g = Graphics.FromImage(img)) {
            g.CompositingQuality = CompositingQuality.HighQuality;
            g.SmoothingMode = SmoothingMode.HighQuality;
            g.Clear(Color.Gray);
            g.DrawString(text, verdanaBold, new SolidBrush(Color.DarkGreen), 0, 0);
            switch(ext) {
                case "jpg":
                case "jpeg":
                    img.Save(context.Response.OutputStream, ImageFormat.Jpeg);
                    break;
                case "gif":
                    img.Save(context.Response.OutputStream, ImageFormat.Gif);
                    break;
            }
        }
    }
    #endregion
}
